import { REMOTE_SCHEME, WEBSERVER } from '../../initializers/constants.js';
function getLocalVideoActivityPubUrl(video) {
    return WEBSERVER.URL + '/videos/watch/' + video.uuid;
}
function getLocalVideoPlaylistActivityPubUrl(videoPlaylist) {
    return WEBSERVER.URL + '/video-playlists/' + videoPlaylist.uuid;
}
function getLocalVideoPlaylistElementActivityPubUrl(videoPlaylist, videoPlaylistElement) {
    return WEBSERVER.URL + '/video-playlists/' + videoPlaylist.uuid + '/videos/' + videoPlaylistElement.id;
}
function getLocalVideoCacheFileActivityPubUrl(videoFile) {
    const suffixFPS = videoFile.fps && videoFile.fps !== -1 ? '-' + videoFile.fps : '';
    return `${WEBSERVER.URL}/redundancy/videos/${videoFile.Video.uuid}/${videoFile.resolution}${suffixFPS}`;
}
function getLocalVideoCacheStreamingPlaylistActivityPubUrl(video, playlist) {
    return `${WEBSERVER.URL}/redundancy/streaming-playlists/${playlist.getStringType()}/${video.uuid}`;
}
function getLocalVideoCommentActivityPubUrl(video, videoComment) {
    return WEBSERVER.URL + '/videos/watch/' + video.uuid + '/comments/' + videoComment.id;
}
function getLocalVideoChannelActivityPubUrl(videoChannelName) {
    return WEBSERVER.URL + '/video-channels/' + videoChannelName;
}
function getLocalAccountActivityPubUrl(accountName) {
    return WEBSERVER.URL + '/accounts/' + accountName;
}
function getLocalAbuseActivityPubUrl(abuse) {
    return WEBSERVER.URL + '/admin/abuses/' + abuse.id;
}
function getLocalVideoViewActivityPubUrl(byActor, video, viewerIdentifier) {
    return byActor.url + '/views/videos/' + video.id + '/' + viewerIdentifier;
}
function getLocalVideoViewerActivityPubUrl(stats) {
    return WEBSERVER.URL + '/videos/local-viewer/' + stats.uuid;
}
function getVideoLikeActivityPubUrlByLocalActor(byActor, video) {
    return byActor.url + '/likes/' + video.id;
}
function getVideoDislikeActivityPubUrlByLocalActor(byActor, video) {
    return byActor.url + '/dislikes/' + video.id;
}
function getLocalVideoSharesActivityPubUrl(video) {
    return video.url + '/announces';
}
function getLocalVideoCommentsActivityPubUrl(video) {
    return video.url + '/comments';
}
function getLocalVideoChaptersActivityPubUrl(video) {
    return video.url + '/chapters';
}
function getLocalVideoLikesActivityPubUrl(video) {
    return video.url + '/likes';
}
function getLocalVideoDislikesActivityPubUrl(video) {
    return video.url + '/dislikes';
}
function getLocalActorFollowActivityPubUrl(follower, following) {
    return follower.url + '/follows/' + following.id;
}
function getLocalActorFollowAcceptActivityPubUrl(actorFollow) {
    return WEBSERVER.URL + '/accepts/follows/' + actorFollow.id;
}
function getLocalActorFollowRejectActivityPubUrl() {
    return WEBSERVER.URL + '/rejects/follows/' + new Date().toISOString();
}
function getLocalVideoAnnounceActivityPubUrl(byActor, video) {
    return video.url + '/announces/' + byActor.id;
}
function getDeleteActivityPubUrl(originalUrl) {
    return originalUrl + '/delete';
}
function getUpdateActivityPubUrl(originalUrl, updatedAt) {
    return originalUrl + '/updates/' + updatedAt;
}
function getUndoActivityPubUrl(originalUrl) {
    return originalUrl + '/undo';
}
function getAbuseTargetUrl(abuse) {
    var _a, _b, _c, _d;
    return ((_b = (_a = abuse.VideoAbuse) === null || _a === void 0 ? void 0 : _a.Video) === null || _b === void 0 ? void 0 : _b.url) ||
        ((_d = (_c = abuse.VideoCommentAbuse) === null || _c === void 0 ? void 0 : _c.VideoComment) === null || _d === void 0 ? void 0 : _d.url) ||
        abuse.FlaggedAccount.Actor.url;
}
function buildRemoteUrl(video, path, scheme) {
    if (!scheme)
        scheme = REMOTE_SCHEME.HTTP;
    const host = video.VideoChannel.Actor.Server.host;
    return scheme + '://' + host + path;
}
function checkUrlsSameHost(url1, url2) {
    const idHost = new URL(url1).host;
    const actorHost = new URL(url2).host;
    return idHost && actorHost && idHost.toLowerCase() === actorHost.toLowerCase();
}
export { getLocalVideoActivityPubUrl, getLocalVideoPlaylistActivityPubUrl, getLocalVideoPlaylistElementActivityPubUrl, getLocalVideoCacheFileActivityPubUrl, getLocalVideoCacheStreamingPlaylistActivityPubUrl, getLocalVideoCommentActivityPubUrl, getLocalVideoChannelActivityPubUrl, getLocalAccountActivityPubUrl, getLocalAbuseActivityPubUrl, getLocalActorFollowActivityPubUrl, getLocalActorFollowAcceptActivityPubUrl, getLocalVideoAnnounceActivityPubUrl, getUpdateActivityPubUrl, getUndoActivityPubUrl, getVideoLikeActivityPubUrlByLocalActor, getLocalVideoViewActivityPubUrl, getVideoDislikeActivityPubUrlByLocalActor, getLocalActorFollowRejectActivityPubUrl, getDeleteActivityPubUrl, getLocalVideoSharesActivityPubUrl, getLocalVideoCommentsActivityPubUrl, getLocalVideoChaptersActivityPubUrl, getLocalVideoLikesActivityPubUrl, getLocalVideoDislikesActivityPubUrl, getLocalVideoViewerActivityPubUrl, getAbuseTargetUrl, checkUrlsSameHost, buildRemoteUrl };
//# sourceMappingURL=url.js.map