import memoizee from 'memoizee';
import { MEMOIZE_LENGTH, MEMOIZE_TTL } from '../initializers/constants.js';
import { TagModel } from '../models/video/tag.js';
import { VideoModel } from '../models/video/video.js';
export async function setVideoTags(options) {
    const { video, tags, transaction } = options;
    const internalTags = tags || [];
    const tagInstances = await TagModel.findOrCreateTags(internalTags, transaction);
    await video.$set('Tags', tagInstances, { transaction });
    video.Tags = tagInstances;
}
async function getVideoDuration(videoId) {
    const video = await VideoModel.load(videoId);
    const duration = video.isLive
        ? undefined
        : video.duration;
    return { duration, isLive: video.isLive };
}
export const getCachedVideoDuration = memoizee(getVideoDuration, {
    promise: true,
    max: MEMOIZE_LENGTH.VIDEO_DURATION,
    maxAge: MEMOIZE_TTL.VIDEO_DURATION
});
//# sourceMappingURL=video.js.map