var TagModel_1;
import { __decorate, __metadata } from "tslib";
import { col, fn, QueryTypes } from 'sequelize';
import { AllowNull, BelongsToMany, Column, CreatedAt, Is, Table, UpdatedAt } from 'sequelize-typescript';
import { VideoPrivacy, VideoState } from '@peertube/peertube-models';
import { isVideoTagValid } from '../../helpers/custom-validators/videos.js';
import { SequelizeModel, throwIfNotValid } from '../shared/index.js';
import { VideoTagModel } from './video-tag.js';
import { VideoModel } from './video.js';
let TagModel = TagModel_1 = class TagModel extends SequelizeModel {
    static findOrCreateTags(tags, transaction) {
        if (tags === null)
            return Promise.resolve([]);
        const uniqueTags = new Set(tags);
        const tasks = Array.from(uniqueTags).map(tag => {
            const query = {
                where: {
                    name: tag
                },
                defaults: {
                    name: tag
                },
                transaction
            };
            return TagModel_1.findOrCreate(query)
                .then(([tagInstance]) => tagInstance);
        });
        return Promise.all(tasks);
    }
    static getRandomSamples(threshold, count) {
        const query = 'SELECT tag.name FROM tag ' +
            'INNER JOIN "videoTag" ON "videoTag"."tagId" = tag.id ' +
            'INNER JOIN video ON video.id = "videoTag"."videoId" ' +
            'WHERE video.privacy = $videoPrivacy AND video.state = $videoState ' +
            'GROUP BY tag.name HAVING COUNT(tag.name) >= $threshold ' +
            'ORDER BY random() ' +
            'LIMIT $count';
        const options = {
            bind: { threshold, count, videoPrivacy: VideoPrivacy.PUBLIC, videoState: VideoState.PUBLISHED },
            type: QueryTypes.SELECT
        };
        return TagModel_1.sequelize.query(query, options)
            .then(data => data.map(d => d.name));
    }
};
__decorate([
    AllowNull(false),
    Is('VideoTag', value => throwIfNotValid(value, isVideoTagValid, 'tag')),
    Column,
    __metadata("design:type", String)
], TagModel.prototype, "name", void 0);
__decorate([
    CreatedAt,
    __metadata("design:type", Date)
], TagModel.prototype, "createdAt", void 0);
__decorate([
    UpdatedAt,
    __metadata("design:type", Date)
], TagModel.prototype, "updatedAt", void 0);
__decorate([
    BelongsToMany(() => VideoModel, {
        foreignKey: 'tagId',
        through: () => VideoTagModel,
        onDelete: 'CASCADE'
    }),
    __metadata("design:type", Array)
], TagModel.prototype, "Videos", void 0);
TagModel = TagModel_1 = __decorate([
    Table({
        tableName: 'tag',
        timestamps: false,
        indexes: [
            {
                fields: ['name'],
                unique: true
            },
            {
                name: 'tag_lower_name',
                fields: [fn('lower', col('name'))]
            }
        ]
    })
], TagModel);
export { TagModel };
//# sourceMappingURL=tag.js.map