import { pick } from '@peertube/peertube-core-utils';
import { getScaleFilter } from '../ffmpeg-utils.js';
import { ffprobePromise, getVideoStreamBitrate, getVideoStreamDimensionsInfo, hasAudioStream } from '../ffprobe.js';
import { addDefaultEncoderGlobalParams, addDefaultEncoderParams, applyEncoderOptions } from './encoder-options.js';
export async function presetVOD(options) {
    const { commandWrapper, input, resolution, fps, scaleFilterValue } = options;
    const command = commandWrapper.getCommand();
    command.format('mp4')
        .outputOption('-movflags faststart');
    addDefaultEncoderGlobalParams(command);
    const probe = await ffprobePromise(input);
    const bitrate = await getVideoStreamBitrate(input, probe);
    const videoStreamDimensions = await getVideoStreamDimensionsInfo(input, probe);
    let streamsToProcess = ['audio', 'video'];
    if (!await hasAudioStream(input, probe)) {
        command.noAudio();
        streamsToProcess = ['video'];
    }
    for (const streamType of streamsToProcess) {
        const builderResult = await commandWrapper.getEncoderBuilderResult(Object.assign(Object.assign({}, pick(options, ['canCopyAudio', 'canCopyVideo'])), { input, inputBitrate: bitrate, inputRatio: (videoStreamDimensions === null || videoStreamDimensions === void 0 ? void 0 : videoStreamDimensions.ratio) || 0, inputProbe: probe, resolution,
            fps,
            streamType, videoType: 'vod' }));
        if (!builderResult) {
            throw new Error('No available encoder found for stream ' + streamType);
        }
        commandWrapper.debugLog(`Apply ffmpeg params from ${builderResult.encoder} for ${streamType} ` +
            `stream of input ${input} using ${commandWrapper.getProfile()} profile.`, { builderResult, resolution, fps });
        if (streamType === 'video') {
            command.videoCodec(builderResult.encoder);
            if (scaleFilterValue) {
                command.outputOption(`-vf ${getScaleFilter(builderResult.result)}=${scaleFilterValue}`);
            }
        }
        else if (streamType === 'audio') {
            command.audioCodec(builderResult.encoder);
        }
        applyEncoderOptions(command, builderResult.result);
        addDefaultEncoderParams({ command, encoder: builderResult.encoder, fps });
    }
}
export function presetCopy(commandWrapper) {
    commandWrapper.getCommand()
        .format('mp4')
        .videoCodec('copy')
        .audioCodec('copy');
}
export function presetOnlyAudio(commandWrapper) {
    commandWrapper.getCommand()
        .format('mp4')
        .audioCodec('copy')
        .noVideo();
}
//# sourceMappingURL=presets.js.map